@@ -1,8 +1,15 @@
 #' Product from collection of spectra
 #'
-#' A method to compute the product of values across members of a collections of
-#' spectra. Computes the product at each wavelength across all the spectra in the
-#' collection returning a spectral object.
+#' Method to compute the "parallel" product of values across members of a
+#' collection of spectra or of a spectral object containing multiple spectra in
+#' long form.
+#'
+#' @details Method specializations compute the product at each wavelength across
+#'   a group of spectra stored in an object of one of the classes defined in
+#'   package 'photobiology'. Omission of NAs is done separately at each
+#'   wavelength. Interpolation is not applied, so all spectra in \code{x} must
+#'   share the same set of wavelengths. An error is triggered if this condition
+#'   is nor fulfilled.
 #'
 #' @param x An R object. Currently this package defines methods for collections of
 #'    spectral objects.
@@ -11,28 +18,29 @@
 #' @param ...	Further arguments passed to or from other methods.
 #'
 #' @return If \code{x} is a collection spectral of objects, such as a
-#'   "filter_mspct" object, the returned object is of same class as the
-#'   members of the collection, such as "filter_spct", containing the product
-#'   of the spectra.
+#'   \code{"filter_mspct"} object, the returned object is of same class as the
+#'   members of the collection, such as \code{"filter_spct"}, containing the
+#'   summary spectrum, with variables with names tagged for summaries other
+#'   than mean or median.
 #'
-#' @note Omission of NAs is done separately at each wavelength. Interpolation is
-#'   not applied, so all spectra in \code{x} must share the same set of
-#'   wavelengths.
+#' @note Objects of classes \code{raw_spct} and \code{cps_spct} can contain data
+#'   from multiple scans in multiple variables or "columns". The methods accept
+#'   as arguments objects of these classes only if spectra contain data for a
+#'   single spectrometer scan. In the case of \code{cps_spct} objects, a single
+#'   column can also contain data from multiple scans spliced into a single
+#'   variable.
 #'
-#'   A product of spectral irradiance or spectral response is no longer a well
-#'   defined physical quanttiy, and these product operations return an object of
-#'   class generic_spct.
-#'
-#'   Objects of classes raw_spct and cps_spct can contain data from multiple
-#'   scans. This functions are implemented for these classes only for the case
-#'   when all member spectra contain data for a single scan, or spliced into a
-#'   single column in the case of cps_spct members.
+#'   The product operation is meaningful only for certain physical
+#'   quantities or bases of expression.
 #'
 #' @seealso See \code{\link[base]{prod}} for the \code{prod()} method used for
 #'   the computations.
 #'
 #' @export
 #'
+#' @examples
+#' s_prod(two_filters.mspct)
+#'
 s_prod <- function(x, na.rm, ...)
   UseMethod("s_prod")
 
@@ -47,6 +55,18 @@ s_prod.default <- function(x, na.rm = FALSE, ...) {
   ifelse(is.any_mspct(x), do.call(class(x[[1]])[1], args = list()), NA)
 }
 
+#' @describeIn s_prod
+#'
+#' @export
+#'
+s_prod.generic_spct <- function(x, na.rm = FALSE, ...) {
+  if (getMultipleWl(x) > 1) {
+    s_prod(subset2mspct(x), na.rm = na.rm, ...)
+  } else {
+    x
+  }
+}
+
 #' @describeIn s_prod
 #'
 #' @export
