@@ -125,14 +125,15 @@ normalize.source_spct <- function(x,
     mspct <- subset2mspct(x,
                           idx.var = getIdFactor(x),
                           drop.idx = FALSE)
-    mspct <- normalize(x = mspct,
-                       range = range,
-                       norm = norm,
-                       unit.out = unit.out,
-                       keep.scaling = keep.scaling,
-                       na.rm = na.rm,
-                       ...)
-    return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+    mspct <-
+      normalize.source_mspct(x = mspct,
+                             range = range,
+                             norm = norm,
+                             unit.out = unit.out,
+                             keep.scaling = keep.scaling,
+                             na.rm = na.rm,
+                             ...)
+    return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
   }
 
   if (is.numeric(keep.scaling)) {
@@ -184,14 +185,15 @@ normalize.response_spct <- function(x,
     mspct <- subset2mspct(x,
                           idx.var = getIdFactor(x),
                           drop.idx = FALSE)
-    mspct <- normalize(x = mspct,
-                       range = range,
-                       norm = norm,
-                       unit.out = unit.out,
-                       keep.scaling = keep.scaling,
-                       na.rm = na.rm,
-                       ...)
-    return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+    mspct <-
+      normalize.response_mspct(x = mspct,
+                               range = range,
+                               norm = norm,
+                               unit.out = unit.out,
+                               keep.scaling = keep.scaling,
+                               na.rm = na.rm,
+                               ...)
+    return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
   }
 
   if (is.numeric(keep.scaling)) {
@@ -247,14 +249,15 @@ normalize.filter_spct <-
       mspct <- subset2mspct(x,
                             idx.var = getIdFactor(x),
                             drop.idx = FALSE)
-      mspct <- normalize(x = mspct,
-                         range = range,
-                         norm = norm,
-                         qty.out = qty.out,
-                         keep.scaling = keep.scaling,
-                         na.rm = na.rm,
-                         ...)
-      return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+      mspct <-
+        normalize.filter_mspct(x = mspct,
+                               range = range,
+                               norm = norm,
+                               qty.out = qty.out,
+                               keep.scaling = keep.scaling,
+                               na.rm = na.rm,
+                               ...)
+      return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
     }
 
     if (is.numeric(keep.scaling)) {
@@ -314,14 +317,15 @@ normalize.reflector_spct <-
       mspct <- subset2mspct(x,
                             idx.var = getIdFactor(x),
                             drop.idx = FALSE)
-      mspct <- normalize(x = mspct,
-                         range = range,
-                         norm = norm,
-                         qty.out = qty.out,
-                         keep.scaling = keep.scaling,
-                         na.rm = na.rm,
-                         ...)
-      return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+      mspct <-
+        normalize.reflector_mspct(x = mspct,
+                                  range = range,
+                                  norm = norm,
+                                  qty.out = qty.out,
+                                  keep.scaling = keep.scaling,
+                                  na.rm = na.rm,
+                                  ...)
+      return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
     }
 
     if (is.numeric(keep.scaling)) {
@@ -360,14 +364,15 @@ normalize.solute_spct <-
       mspct <- subset2mspct(x,
                             idx.var = getIdFactor(x),
                             drop.idx = FALSE)
-      mspct <- normalize(x = mspct,
-                         range = range,
-                         norm = norm,
-#                         qty.out = qty.out,
-                         keep.scaling = keep.scaling,
-                         na.rm = na.rm,
-                         ...)
-      return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+      mspct <-
+        normalize.solute_mspct(x = mspct,
+                               range = range,
+                               norm = norm,
+                               #                         qty.out = qty.out,
+                               keep.scaling = keep.scaling,
+                               na.rm = na.rm,
+                               ...)
+      return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
     }
 
     if (is.numeric(keep.scaling)) {
@@ -404,13 +409,14 @@ normalize.raw_spct <-
       mspct <- subset2mspct(x,
                             idx.var = getIdFactor(x),
                             drop.idx = FALSE)
-      mspct <- normalize(x = mspct,
-                         range = range,
-                         norm = norm,
-                         keep.scaling = keep.scaling,
-                         na.rm = na.rm,
-                         ...)
-      return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+      mspct <-
+        normalize.raw_mspct(x = mspct,
+                            range = range,
+                            norm = norm,
+                            keep.scaling = keep.scaling,
+                            na.rm = na.rm,
+                            ...)
+      return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
     }
 
     if (is.numeric(keep.scaling)) {
@@ -448,13 +454,14 @@ normalize.cps_spct <-
       mspct <- subset2mspct(x,
                             idx.var = getIdFactor(x),
                             drop.idx = FALSE)
-      mspct <- normalize(x = mspct,
-                         range = range,
-                         norm = norm,
-                         keep.scaling = keep.scaling,
-                         na.rm = na.rm,
-                         ...)
-      return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+      mspct <-
+        normalize.cps_mspct(x = mspct,
+                            range = range,
+                            norm = norm,
+                            keep.scaling = keep.scaling,
+                            na.rm = na.rm,
+                            ...)
+      return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
     }
 
     if (is.numeric(keep.scaling)) {
@@ -501,14 +508,15 @@ normalize.generic_spct <-
       mspct <- subset2mspct(x,
                             idx.var = getIdFactor(x),
                             drop.idx = FALSE)
-      mspct <- normalize(x = mspct,
-                         range = range,
-                         norm = norm,
-                         col.names = col.names,
-                         keep.scaling = keep.scaling,
-                         na.rm = na.rm,
-                         ...)
-      return(rbindspct(mspct, idfactor = FALSE, attrs.simplify = TRUE))
+      mspct <-
+        normalize.generic_mspct(x = mspct,
+                                range = range,
+                                norm = norm,
+                                col.names = col.names,
+                                keep.scaling = keep.scaling,
+                                na.rm = na.rm,
+                                ...)
+      return(rbindspct(mspct, idfactor = getIdFactor(x), attrs.simplify = TRUE))
     }
 
     normalize_spct(spct = x,
@@ -705,6 +713,59 @@ normalize.cps_mspct <- function(x,
           .paropts = .paropts)
 }
 
+#' @describeIn normalize Normalize the members of a solute_mspct object.
+#'
+#' @export
+#'
+normalize.solute_mspct <- function(x,
+                                   ...,
+                                   range = x,
+                                   norm = "max",
+                                   keep.scaling = FALSE,
+                                   na.rm = FALSE,
+                                   .parallel = FALSE,
+                                   .paropts = NULL) {
+
+  if (!length(x)) return(x) # class of x in no case changes
+
+  msmsply(x,
+          normalize.solute_spct,
+          range = range,
+          norm = norm,
+          keep.scaling = keep.scaling,
+          na.rm = na.rm,
+          ...,
+          .parallel = .parallel,
+          .paropts = .paropts)
+}
+
+#' @describeIn normalize Normalize the members of a solute_mspct object.
+#'
+#' @export
+#'
+normalize.generic_mspct <- function(x,
+                                    ...,
+                                    range = NULL,
+                                    norm = "max",
+                                    col.names,
+                                    keep.scaling = FALSE,
+                                    na.rm = FALSE,
+                                    .parallel = FALSE,
+                                    .paropts = NULL) {
+
+  if (!length(x)) return(x) # class of x in no case changes
+
+  msmsply(x,
+          normalize, # members can be heterogeneous
+          range = range,
+          norm = "max",
+          keep.scaling = keep.scaling,
+          na.rm = na.rm,
+          ...,
+          .parallel = .parallel,
+          .paropts = .paropts)
+}
+
 # PRIVATE -----------------------------------------------------------------
 
 #' @keywords internal
